
local CommChannel = CommChannel("1.0")
local IFrameFactory = IFrameFactory("1.0")

local HealSyncDock = { }
local HealSyncUnit = { }

local iface = IFrameManager:Interface()
function iface:getName(frame)
	return string.gsub(frame:GetName(), "_", " ")
end

local function CreateDock(unitID, label)
	this = CreateFrame("Frame", "HealSync__"..label, UIParent)
	this:SetWidth(256)
	this:SetHeight(20)
	this:SetPoint("CENTER", UIParent)
	this:SetMovable(true)
	this:SetUserPlaced(true)
	
	HealSyncDock[unitID] = this	
	IFrameManager:Register(this, iface)
end

CreateDock("player", "Player")
CreateDock("target", "Target")

local function UpdateDock(unitID)
	local parent = HealSyncDock[unitID]
	
	for name, info in pairs(HealSyncUnit) do
		if (info[1] == UnitName(unitID) and info[2]) then
			local spell, rank, display, icon, startTime, endTime = UnitCastingInfo(info[2])
			if (spell) then
				local frame = IFrameFactory:Create("HealSync", "Button")
				frame.unitID = info[2]
				frame.caster:SetText(name)
				frame.spell:SetText(display)
				frame.bar:SetMinMaxValues(startTime, endTime)
				if (name == UnitName("player")) then
					frame.bar:SetStatusBarColor(1.0, 0.0, 0.0, 0.8)
				else
					local _, class = UnitClass(info[2])
					local color = RAID_CLASS_COLORS[class or "PRIEST"]
					frame.bar:SetStatusBarColor(color.r, color.g, color.b, 0.8)
				end
				frame:ClearAllPoints()
				if (parent == HealSyncDock[unitID]) then
					frame:SetPoint("TOP", parent, "TOP", 0, 0)
				else
					frame:SetPoint("TOP", parent, "BOTTOM", 0, 2)
				end
		
				parent = frame
			end
		end
	end
end

local function HealSyncUpdate(name)
	if (name == UnitName("player") or name == UnitName("target")) then
		IFrameFactory:Clear("HealSync", "Button")
	
		UpdateDock("player")
		UpdateDock("target")
	end
end

local iface = { }
local module = CommChannel:Create("RAID", "HSC", iface)

function iface:T(target)
	HealSyncUnit[arg4] = HealSyncUnit[arg4] or { }
	HealSyncUnit[arg4][1] = target
	HealSyncUpdate(target)
end

local function onEvent(self, event, ...)
	if (event == "PLAYER_ENTERING_WORLD") then
		HealSyncUpdate()
	elseif (event == "PLAYER_TARGET_CHANGED") then
		HealSyncUpdate()
	elseif (event == "UNIT_SPELLCAST_SENT") then
		local target = select(4, ...)
		if (#target > 0) then
			module:Call("T", target);
		end
	elseif (event == "UNIT_SPELLCAST_START") then
		local unitID = select(1, ...)
		local unit = HealSyncUnit[UnitName(unitID)]
		if (unit) then
			unit[2] = unitID
			HealSyncUpdate(unit[1])
		end
	elseif (event == "UNIT_SPELLCAST_DELAYED") then
		local unitID = select(1, ...)
		local unit = HealSyncUnit[UnitName(unitID)]
		if (unit) then
			unit[2] = unitID
			HealSyncUpdate(unit[1])
		end
	elseif (event == "UNIT_SPELLCAST_STOP") then
		local unitID = select(1, ...)
		local unit = HealSyncUnit[UnitName(unitID)]
		if (unit) then
			unit[2] = nil
			HealSyncUpdate(unit[1])
		end
	end
end

this = CreateFrame("Frame")
this:RegisterEvent("PLAYER_ENTERING_WORLD")
this:RegisterEvent("PLAYER_TARGET_CHANGED")
this:RegisterEvent("UNIT_SPELLCAST_SENT")
this:RegisterEvent("UNIT_SPELLCAST_START")
this:RegisterEvent("UNIT_SPELLCAST_DELAYED")
this:RegisterEvent("UNIT_SPELLCAST_STOP")
this:SetScript("OnEvent", onEvent)
